!
! SESDmtrx --- Calculate matrix representation of the radial matrix
!       element of the dipmat operator for smooth exterior complex
!       scaling (SES)
! Christian Buth
! 25 May 2006
!
!
! ngrd  : number of grid points of the quadratic mesh
! xh    : spacing of points in the linear x=sqrt(r)-mesh
! egfxn : eigenfunctions of the atomic Hamiltonian with the
!         Herman-Skillman potential represented
!         in the finite element basis
! theta : complex scaling angle
!

      subroutine SESDmtrx(dipmat, nstate, nl, egfxn, ngrd, xh)
      use quadrature
      use SESpath
      implicit none

      integer nstate, nl, ngrd
      integer i, j, k, l1, l2
      real(dblprec) egfxn(ngrd,nstate,nl)
      real(dblprec), allocatable :: rework(:), imwork(:)
      real(dblprec) r, x, xh, numinterr, re1, im1, re2, im2
      complex(dblprec) dipmat(nstate,nl,nstate,nl)
      complex(dblprec) value

      allocate(rework(ngrd), imwork(ngrd))
      numinterr = 0.0D0

      ! Sample the radial part of the dipole operator on the
      ! numeric (quadratic) integration grid
      do l2 = 1, nl
        do k = 1, nstate
          do l1 = 1, nl
            do j = 1, nstate
              do i = 1, ngrd
                x = xh * real(i, dblprec)
                rework(i) = egfxn(i,j,l1) * egfxn(i,k,l2)         ! Eigenfunction values
                rework(i) = 2.0D0 * x * rework(i)                 ! Integration measure
                imwork(i) = rework(i)
                value = pathFc(x**2)
                rework(i) = rework(i) * real (value, dblprec)
                imwork(i) = imwork(i) * aimag(value)
              end do

              ! Numerical integration
!               re1 = midpointrule (ngrd,xh,rework)
!               im1 = midpointrule (ngrd,xh,imwork)
!               re2 = trapezoidrule(ngrd,xh,rework)
!               im2 = trapezoidrule(ngrd,xh,imwork)
              re1 = threeeighthsrule(ngrd,xh,rework)
              im1 = threeeighthsrule(ngrd,xh,imwork)
              re2 = simpsonsrule    (ngrd,xh,rework)
              im2 = simpsonsrule    (ngrd,xh,imwork)
              r = (re1 - re2)**2 + (im1 - im2)**2
              if (r > numinterr) then
                numinterr = r
              end if
              dipmat(j,l1,k,l2) = cmplx(re2, im2, dblprec)
!               dipmat(k,l2,j,l1) = dipmat(j,l1,k,l2)                ! dipmat is complex symmetric
            end do
          end do
        end do
      end do

      write (*,*) '        Maximum error in numerical integration ', sqrt(numinterr)
      deallocate(rework, imwork)
      end subroutine SESDmtrx
