!
! twoph --- dressed Rydberg calculation
! Christian Buth
! 27 Jun 2007
!
! Version V1.0.0: 11 Sep 2007 (Heureka!  The one-photon anbsorption cross
!       section gives the same result as dreyd)
!


      program twoph
      use mathtl
      use physconst
      use floquetmat
      implicit none

      type dipstate
        integer l1, l2
        complex(dblprec), pointer :: trans(:,:,:)
      end type dipstate

      ! Number of x-ray photon energy grid points desired
      integer nx
      ! Orbital angular momentum quantum numbers for the
      ! initial one-particle state
      integer li
      ! Range of orbital angular momenta taken into account
      ! in Floquet calculation; s, p, and d orbitals should
      ! always be included
      integer lmin, lmax
      ! Occupied orbitals in the s: nocc(0), p: nocc(1), ... shells
      integer, parameter :: maxlocc = 50
      integer nocc(0:maxlocc), nshellocc(0:maxlocc)
      ! Number of l blocks
      integer nl
      ! Number of atomic eigenvectors per l and photon block
      integer nvec

      integer dimen, nvlnc, i, j, k, l, init, nmain,  &
     &          ralmin, ralmax
                ! evinit,

      integer, parameter :: uxsect = 97    ! Energy dependent cross section file

      real(dblprec) omegaX, E_ini, Gamma, Xmin, Xmax, Delta, &
     &          sigtot, sigtot2, Eground
      real(dblprec), allocatable :: eval(:,:)

      complex(dblprec), allocatable ::  &
     &          SESWatom(:,:,:),                    &
     &          SESdipatm(:,:,:,:),                 &
     &          SESpotatm(:,:,:),                   &
     &          SESfTfatm(:,:,:),                   &
     &          evec(:,:,:,:),                      &
     &          spectrum(:,:,:)

      type(dipstate) intermediate(2), final(4)

      namelist/TwoPhXSect/ nx, li, lmin, lmax, nvec, nocc, Xmin,  &
     &          Xmax, E_ini, Gamma, init

      ! Initialization
      write (*,*) 'twoph V1.1.0, two-photon cross section calculations'
      write (*,*) '(c) 2007 Christian Buth'
      write (*,*)

      ! Default values for parameters
      nx = 10001
      li = 0
      lmin = 0 ; lmax = 2
      nl = lmax - lmin + 1
      nvec = 50
      nocc(:) = 0
      Xmin = 14310.0D0
      Xmax = 16410.0D0
      E_ini = -14326.0D0
      Gamma = 2.7D0
      init = 1
      nmain = 1

      read(*,nml=TwoPhXSect)

      nl = lmax - lmin + 1
      if (lmin < 0 .or. lmin > maxlocc .or. lmax < 0 .or. lmax > maxlocc) then
        write (*,*)
        write (*,*) 'twoph: illegal parameter range for lmin and lmax'
        write (*,*) 'Range of orbital angular momenta taken into account ', &
     &              'in Floquet calculation;'
        write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &              lmin, 'lmax =', lmax
        stop 'twoph: illegal parameter range'
      end if

      ! Echo input
      write (*,*) 'Range of orbital angular momenta taken into account ', &
     &            'in Floquet calculation;'
      write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &            lmin, 'lmax =', lmax
      write (*,*) 'Number of orbitals occupied in the ground state:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)
      write (*,*) 'Number of l blocks: nl =', nl
      write (*,*) 'Number of atomic eigenvectors per l and photon block: nvec =', nvec

      dimen = nvec
      write (*,*) 'Examine the photoabsorption of state', init
      write (*,*) 'Experimental (negative) binding energy of the electron in ',  &
     &            'the state in eV', E_ini
      write (*,*) 'Experimental decay width of the hole in eV', Gamma

      write (*,*) 'Number of x-ray photon energy grid points: nx =', nx
      write (*,*) 'Desired x-ray photon energy range is from Xmin (in eV)', Xmin,  &
     &            'to Xmax (in eV)', Xmax

      allocate(eval(nvec,lmin:lmax))
      allocate( SESWatom(nvec,nvec,lmin:lmax),             &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),  &
     &          SESpotatm(nvec,nvec,lmin:lmax),            &
     &          SESfTfatm(nvec,nvec,lmin:lmax))

      !ccccc x-ray parameters ccccccccccccccccccccccccccccccccccccccccc
      ! the desired x-ray photon energy range is from Xmin (in eV)
      ! to Xmax (in eV); conversion to hartree
      Xmin = Xmin * eVToHartree
      Xmax = Xmax * eVToHartree

      ! ccccc atomic parameters cccccccccccccccccccccccccccccccccccccccc
      ! Experimental (negative) binding energy of Kr 1s electron in eV
      ! (the minus sign is needed)
      E_ini = E_ini * eVToHartree
      ! Experimental decay width of the hole in eV
      Gamma = Gamma * eVToHartree
      write (*,*)

      ! Read atomic one-electron eigenenergies (eval) and dipole matrix elements
      ! (SESDatom); these quantities have to be calculated, before running this code,
      ! using oneel
      call readatom(nl, lmin, lmax, nvec, nvlnc, eval, SESdipatm,  &
     &              SESfTfatm, SESpotatm, SESWatom, li, init, Eground)
      write(*,*) 'Number of the s shell of the valence electron, nvlnc =', nvlnc
!       E_ini = eval(init, li)
      write (*,*) 'Hartree-Fock-Slater (negative) binding energy of the electron in ',  &
     &            'the initial state in eV', eval(init, li) * HartreeToeV

      ! Recalculate the nocc array after subtraction of the
      ! core orbitals nvlnc
      nshellocc(:) = 0    ! S orbitals are the first orbitals of the periodic table
      nshellocc(1) = 1    ! P orbitals start in the second row of the periodic table
      nshellocc(2) = 2    ! D orbitals start in the third row of the periodic table
      nshellocc(3) = 3    ! F orbitals start in the fourth row of the periodic table
      do l = lmin, lmax
        j = nvlnc - 1 - nshellocc(l)
        if (nocc(l) > 0 .and. j > 0) then
          nocc(l) = nocc(l) - j
          if (nocc(l) < 0) then
            nocc(l) = 0
          end if
        end if
      end do
      write (*,*) 'Occupation pattern in the ground state after subtraction of ',  &
     &            'the core orbitals nvlnc:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)

      ! Decouple states that are lower in energy than the valence
      ! state under consideration. Such states are assumed to be fully
      ! occupied. Electrons in the valence shell of angular momentum
      ! "li" must therefore not be allowed to make transitions into
      ! any lower lying levels. (The lowest-energy state in the file
      ! atom.dat is always an s state.)

      ! Core orbitals, including the 1s orbital (which is only singly
      ! occupied after absorbing the x-ray photon), are decoupled in
      ! the Floquet treatment of the Rydberg manifold
      do l = lmin, lmax
        do k = 1, nocc(l)
          do j = lmin, lmax
            do i = 1, nocc(j)
              SESdipatm(i,j,k,l) = (0.0D0, 0.0D0)
              SESdipatm(k,l,i,j) = (0.0D0, 0.0D0)
            end do
          end do
        end do
      end do

      ! Compute atomic levels plus CAP
      ralmin = min(abs(li-2), abs(li-1), li)
      ralmax = li+2
      allocate(spectrum(dimen,    -li:li, ralmin:ralmax),  &
     &         evec(dimen, dimen, -li:li, ralmin:ralmax))
      write (*,*)
      write (*,*) 'Computing atomic energy levels plus CAP: '
      do l = ralmin, ralmax
        write (*,*)
        write (*,*) '  Diagonalizing for angular momentum', l
        call dothediag(l)
      end do

      ! The initial state
      write (*,*)
      write (*,*) 'Initial state l=', li

      ! Generate cascade of intermediate states
      intermediate(1)%l1 = li ; intermediate(1)%l2 = li - 1
      intermediate(2)%l1 = li ; intermediate(2)%l2 = li + 1
      do l=1, 2
        if (intermediate(l)%l2 < 0) then
          cycle
        end if
        write (*,*)
        write (*,*) 'Intermediate state No ', l, ' with l=', intermediate(l)%l2
        ! The full diagonalization does not directly provide
        ! transition matrix elements.  They need to be computed
        ! separately here.
        call halfcalconetrans(intermediate(l))
      end do

      ! Generate cascade of final states
      final(1)%l1 = li - 1 ; final(1)%l2 = li - 2
      final(2)%l1 = li - 1 ; final(2)%l2 = li
      final(3)%l1 = li + 1 ; final(3)%l2 = li
      final(4)%l1 = li + 1 ; final(4)%l2 = li + 2
      do l=1, 4
        if (final(l)%l1 < 0 .or. final(l)%l2 < 0) then
          cycle
        end if
        write (*,*)
        write (*,*) 'Final state No ', l, ' with l1=', final(l)%l1, 'and l2=', final(l)%l2
        ! The full diagonalization does not directly provide
        ! transition matrix elements.  They need to be computed
        ! separately here.
        call calctwotrans(final(l))
      end do
      deallocate(evec)

      ! Calculate the total cross section depending on the
      ! x-ray photon energy
      write (*,*)
      write (*,*) 'Total two-photon cross section depending on the x-ray photon energy ',  &
     &          'from Xmin =', Xmin * HartreeToeV, 'to Xmax =', Xmax *          &
     &          HartreeToeV, 'in nx =', nx, ' steps'
      open(unit=uxsect, file='xsect.dat', status='replace', action='write',  &
     &          form='formatted', access='sequential')
      ! Step size for the calculation of the x-ray photoabsorption cross section
      Delta = (Xmax - Xmin) / real(nx-1, dblprec)
      omegaX = Xmin
      do i = 1, nx
        ! The photoabsorption cross section is calculated in atomic units;
        ! the result is written to the file in kilobarns
        ! the result is written to the file in cm^4 s
        sigtot  = modularTwoPhotonX(omegaX)
        sigtot2 = singleTwoPhotonX(omegaX)
        write(uxsect,*) omegaX * HartreeToeV, sigtot  * t0ToSecond * BohrToCentimeter**4,  &
     &                                        sigtot2 * t0ToSecond * BohrToCentimeter**4
        sigtot = onePhotonX(omegaX)
        write(120,*) omegaX * HartreeToeV, sigtot * cross_section_AUtoBarn / 1000.0D0
        omegaX = omegaX + Delta
      end do
      close(uxsect)
      deallocate(SESWatom, SESdipatm, SESpotatm, SESfTfatm, eval)

      write (*,*)
      write (*,*) '???'
      stop 'Happy end!'


      contains

!
! twoPhotonX --- calculate the sum for the cross section over all
!       dressed states
!
! Note that here the experimental E_ini is subtracted!
!

      real(dblprec) function singleTwoPhotonX(omegaX)
      implicit none

      integer n1, l1, lf, m1, l2, n2
      real(dblprec), intent(in) :: omegaX
      complex(dblprec) xsect, K_IF, E_IF, kadd, E_IM

!         stop 'no to be used presently'
!         write (313,*) omegaX

      xsect = 0
      do l1=1, 4
        if (final(l1)%l1 < 0 .or. final(l1)%l2 < 0) then
          cycle
        end if
        lf = min(li, final(l1)%l1, final(l1)%l2)
        do m1 = -lf, lf
          do n1=1, nvec

            kadd = 0
            do l2=1, 2
              ! Only permit valid records
              if (intermediate(l2)%l1 < 0 .or. intermediate(l2)%l2 < 0) then
                cycle
              end if
              ! Angular momenta do not match
              if (final(l1)%l1 /= intermediate(l2)%l2) then
                cycle
              end if
              ! Ensure valid input parameters
              if (intermediate(l2)%l1 < abs(m1) .or. intermediate(l2)%l2 < abs(m1) .or.  &
     &            final(l1)%l1        < abs(m1) .or. final(l1)%l2        < abs(m1)) then
                write (*,*) intermediate(l2)%l1, m1, intermediate(l2)%l2, final(l1)%l1, final(l1)%l2
                stop 'twoPhotonX: invalid range of magnetic quantum number!'
              end if

              do n2=1, nvec
!                E_IM = cmplx(E_ini + omegaX, 0.0D0, dblprec)
                E_IM = cmplx(E_ini + omegaX, -1.3252083039420256D-7, dblprec)
!                E_IM = spectrum(init,m1,li) + cmplx(omegaX, 0.0D0, dblprec)
                E_IM = E_IM - spectrum(n2,m1,intermediate(l2)%l2)
                ! A la McGuire; avoid divergence of matrix elements by modifying
                ! their real part slightly
                if (abs(real(E_IM)) < 1.0D-2) then
                  write (313,*) n1, l1, m1, n2, l2, E_IM, (E_IM - 3.0D0 * eVToHartree)
                  E_IM = E_IM - 3.0D0 * eVToHartree
                end if
                K_IF = final(l1)%trans(n1,n2,m1) * intermediate(l2)%trans(n2,init,m1)
                kadd = kadd + K_IF / E_IM
              end do
            end do

!            E_IF = cmplx(E_ini + 2.0D0 * omegaX, 0.0D0, dblprec)
            E_IF = cmplx(E_ini + 2.0D0 * omegaX, -1.3252083039420256D-7, dblprec)
!            E_IF = spectrum(init,m1,li) + cmplx(2.0D0 * omegaX, 0.0D0, dblprec)
!            write (303,*) E_ini, spectrum(init,m1,li)
            E_IF = E_IF - spectrum(n1,m1,final(l1)%l2)
            xsect = xsect + kadd**2 / E_IF
          end do
        end do
      end do

      singleTwoPhotonX = -16.0D0 * (M_PI * fine_structure_constant  &
     &          * omegaX)**2 * aimag(xsect)
      end function singleTwoPhotonX


      real(dblprec) function modularTwoPhotonX(omegaX)
      implicit none

      integer n1, l1, lf, m1
      real(dblprec), intent(in) :: omegaX
      complex(dblprec) xsect, K_IF, E_IF

      xsect = 0
      do l1=1, 4
        if (final(l1)%l1 < 0 .or. final(l1)%l2 < 0) then
          cycle
        end if
        lf = min(final(l1)%l1, final(l1)%l2, li)

        do n1=1, nvec
          do m1 = -lf, lf
            K_IF = getKIF(omegaX, n1, final(l1), m1)
!            if (K_IF /= 0) then
              E_IF = spectrum(n1,m1,final(l1)%l2)
              E_IF = cmplx(E_ini + 2.0D0 * omegaX, 0.0D0, dblprec) - E_IF
              xsect = xsect + K_IF**2 / E_IF

!               if (n1 == 6 .and. final(l1)%l2 == 3 .and. m1 == 0) then
!                 write (414,*) omegaX * HartreeToeV, real(getKIF(omegaX, n1, final(l1), m1)**2),  &
!      &                  aimag(getKIF(omegaX, n1, final(l1), m1)**2),  &
!      &                  real(getKIF(omegaX, n1, final(l1), m1)),  &
!      &                  aimag(getKIF(omegaX, n1, final(l1), m1))
!               end if

!            end if
          end do
        end do
      end do

      modularTwoPhotonX = -16.0D0 * (M_PI * fine_structure_constant  &
     &          * omegaX)**2 * aimag(xsect)
      end function modularTwoPhotonX


!
! getKIF --- compute the transition matrix element
!

      complex(dblprec) function getKIF(omegaX, n_K, state, m_K)
      implicit none

      integer l2, n2, n_K, m_K
      real(dblprec), intent(in) :: omegaX
      complex(dblprec) K_IF, kadd, E_IM, divCorr
      type(dipstate), intent(inout) :: state

      ! Examine whether a divergence correction is necessary
      divCorr = 0
            goto 200
      do l2=1, 2
        ! Only permit valid records
        if (intermediate(l2)%l1 < 0 .or. intermediate(l2)%l2 < 0 .or.  &
     &    state%l1              < 0 .or. state%l2            < 0) then
          cycle
        end if
        ! Angular momenta do not match
        if (state%l1 /= intermediate(l2)%l2) then
          cycle
        end if
        ! Ensure valid input parameters
        if (intermediate(l2)%l1 < abs(m_K) .or. intermediate(l2)%l2 < abs(m_K) .or.  &
     &      state%l1            < abs(m_K) .or. state%l2            < abs(m_K)) then
          write (*,*) intermediate(l2)%l1, m_K, intermediate(l2)%l2, state%l1, state%l2
          stop 'getKIF: invalid range of magnetic quantum number!'
        end if

        do n2=1, nvec
          E_IM = cmplx(E_ini + omegaX, 0.0D0, dblprec)
          E_IM = E_IM - spectrum(n2,m_K,intermediate(l2)%l2)
          if (abs(real(E_IM)) < 1.0D-4) then
            write (313,*) n2, l2, E_IM, (E_IM - 3.0D0 * eVToHartree)
            divCorr = 1.0D0 * eVToHartree
            goto 200
          end if
        end do
      end do

 200  continue

      K_IF = 0
      do l2=1, 2
        ! Only permit valid records
        if (intermediate(l2)%l1 < 0 .or. intermediate(l2)%l2 < 0 .or.  &
     &    state%l1              < 0 .or. state%l2            < 0) then
          cycle
        end if
        ! Angular momenta do not match
        if (state%l1 /= intermediate(l2)%l2) then
          cycle
        end if
        ! Ensure valid input parameters
        if (intermediate(l2)%l1 < abs(m_K) .or. intermediate(l2)%l2 < abs(m_K) .or.  &
     &      state%l1            < abs(m_K) .or. state%l2            < abs(m_K)) then
          write (*,*) intermediate(l2)%l1, m_K, intermediate(l2)%l2, state%l1, state%l2
          stop 'getKIF: invalid range of magnetic quantum number!'
        end if

        do n2=1, nvec
          E_IM = cmplx(E_ini + omegaX, 0.0D0, dblprec)
          E_IM = E_IM - spectrum(n2,m_K,intermediate(l2)%l2)
          if (abs(real(E_IM)) < 1.0D-3) then
            write (313,*) n2, intermediate(l2)%l2, E_IM, (E_IM - 3.0D0 * eVToHartree)
          end if
          E_IM = E_IM + divCorr
          kadd = state%trans(n_K,n2,m_K) * intermediate(l2)%trans(n2,init,m_K)
          kadd = kadd / E_IM
          K_IF = K_IF + kadd
        end do
      end do

      getKIF = K_IF
      end function getKIF


!
! onePhotonX --- calculate the sum for the cross section over all
!       dressed states
!
! Note that here the experimental E_ini is subtracted!
!

      real(dblprec) function onePhotonX(omegaX)
      implicit none

      integer l2, n2, mf, lf
      real(dblprec), intent(in) :: omegaX
      complex(dblprec) xsect, E_IM, TIF

      xsect = 0
      do l2=1, 2
        if (intermediate(l2)%l1 < 0 .or. intermediate(l2)%l2 < 0) then
          cycle
        end if
        lf = min(intermediate(l2)%l1, intermediate(l2)%l2)

        do n2=1, nvec
          do mf = -lf, lf
            E_IM = spectrum(n2,mf,intermediate(l2)%l2)
            E_IM = E_IM - cmplx(E_ini + omegaX, 0.0D0, dblprec)
            TIF = intermediate(l2)%trans(n2,init,mf)
            xsect = xsect + TIF**2 / E_IM
          end do
        end do
      end do

      onePhotonX = 2.0D0 * 4.0D0 * M_PI * fine_structure_constant * omegaX * aimag(xsect)
      end function onePhotonX


!
! dothediag --- calculate the eigenbasis of the zero-order hamiltonian
!       which is given by the Herman-Skillman hamiltonian plus the CAP
!

      subroutine dothediag(angmom)
      implicit none

      integer mf, lf, lm, nphmin, nphmax, j
      real(dblprec) cconst, omegaL, Eground
      integer, intent(in) :: angmom

      lf = angmom
      lm = min(li, angmom)
      spectrum(:,:,angmom) = 0

      ! There is no laser coupling
      cconst = 0.0D0
      omegaL = 0.0D0            ! No laser dressing is implemented
      nphmin = 0
      nphmax = 0
      Eground = 0.0D0           ! In twoph not used

      ! Treat all magnetic sublevels of the angular momentum lm
      do mf = -lm, lm
        write(*,*) 'Diagonalizing the complex symmetric Hamiltonian ',  &
     &             'with dimension =', dimen, ', lf = ', lf, ', mf = ', mf
        lmin = lf
        lmax = lf
        call hamiltonian(dimen, nmain, nphmin, nphmax, lmin, lmax, nvec,  &
     &          mf, cconst, eval(:,lmin:lmax), Eground, omegaL,           &
     &          SESfTfatm(:,:,lmin:lmax), SESpotatm(:,:,lmin:lmax),       &
     &          SESWatom(:,:,lmin:lmax), SESdipatm(:,lmin:lmax,:,lmin:lmax))
        call fulldiagfloquet('V', dimen, spectrum(:,mf:mf,angmom:angmom),  &
     &          evec(1:nvec, 1:nvec, mf:mf, angmom:angmom))

        ! Subtract decay width of core hole from excited states eigenvalues.
        ! Only one-particle states unoccupied in the ground state are regarded.
        ! Attention!!  I assume that the eigenvalues are still sorted with
        ! respect to an increasing real part.
        do j = nocc(lf)+1, dimen
          spectrum(j,mf,angmom) = spectrum(j,mf,angmom)  &
     &          - cmplx(0.0D0, 0.5D0*Gamma, dblprec)
!          write (304,*) j,mf,angmom,spectrum(j,mf,angmom)
        end do

!        write (*,*) '============= E_ini ', E_ini
!        do j = 1, dimen
!          if (abs(real(spectrum(j,mf,angmom)) - E_ini) < 1.0D-2) then
!            write (*,*) '=======================', j, mf, angmom, E_ini, spectrum(j,mf,angmom)
!            write (306,*)
!            write (306,*) '=======================', j, mf, angmom, E_ini, spectrum(j,mf,angmom)
!            do i = 1, dimen
!              write (306,*) evec(i, j, mf, angmom)
!            end do
!            evinit = j
!          end if
!        end do
!
!     Instead!!!!!!!!!!!!!!!
!        evinit = init
      end do
      end subroutine dothediag


!
! calconetrans --- transformation of the transition dipole moments between
!       a one-particle state in the atomic ground state and all laser
!       dressed atomic states from the finite element basis to the
!       eigenvector basis of the Floquet matrix
!

!      subroutine fullcalconetrans(state)
!      implicit none
!
!      integer ls, ms, i, j, k
!      type(dipstate), intent(inout) :: state
!
!      ls = min(state%l1, state%l2)
!      allocate(state%trans(nvec, init:init, -ls:ls))
!      state%trans(:,:,:) = 0
!
!      ! Treat all magnetic sublevels of the angular momentum ls
!      do ms = -ls, ls
!        do k = 1, nvec
!          state%trans(k,init,ms) = 0
!          do i = 1, nvec
!            do j = 1, nvec
!              state%trans(k,init,ms) = state%trans(k,init,ms) + evec(i,k,ms,state%l2)  &
!     &             * evec(j,evinit,ms,state%l1) * SESdipatm(i,state%l2,j,state%l1)     &
!     &             * cmplx(sqrt(  real(2 * state%l1 + 1, dblprec)            &
!     &                          / real(2 * state%l2 + 1, dblprec))           &
!     &                     * clebsch(state%l1,1,state%l2, ms,0,ms)           &
!     &                     * clebsch(state%l1,1,state%l2, 0,0,0), 0.0D0, dblprec)
!            end do
!          end do
!        end do
!      end do
!      end subroutine fullcalconetrans


      subroutine halfcalconetrans(state)
      implicit none

      integer lf, mf, i, j
      type(dipstate), intent(inout) :: state

      lf = min(state%l1, state%l2)
      allocate(state%trans(nvec,init:init,-lf:lf))
      state%trans(:,:,:) = 0

      ! Treat all magnetic sublevels of the angular momentum lf
      do mf = -lf, lf
        do j = 1, nvec
          state%trans(j,init,mf) = 0
          do i = 1, nvec
            state%trans(j,init,mf) = state%trans(j,init,mf)                 &
     &          + evec(i,j,mf,state%l2) * SESdipatm(i,state%l2,init,state%l1)  &
     &          * cmplx(sqrt(  real(2 * state%l1 + 1, dblprec)              &
     &                       / real(2 * state%l2 + 1, dblprec))             &
     &                  * clebsch(state%l1,1,state%l2, mf,0,mf)             &
     &                  * clebsch(state%l1,1,state%l2, 0,0,0), 0.0D0, dblprec)
           end do
        end do
      end do
      end subroutine halfcalconetrans


!
! calctwotrans --- transformation of the transition dipole moments between
!       a one-particle state in the atomic ground state and intermediate
!       atomic states from the finite element basis to the eigenvector
!       basis of the Floquet matrix
!

      subroutine calctwotrans(state)
      implicit none

      integer ls, ms, i, j, k, l
      type(dipstate), intent(inout) :: state

      ls = min(li, state%l1, state%l2)
      allocate(state%trans(nvec, nvec, -ls:ls))
      state%trans(:,:,:) = 0

      ! Treat all magnetic sublevels of the angular momentum ls
      do ms = -ls, ls
        do k = 1, nvec
          do l = 1, nvec
            state%trans(k,l,ms) = 0
            do i = 1, nvec
              do j = 1, nvec
                state%trans(k,l,ms) = state%trans(k,l,ms) + evec(i,k,ms,state%l2) &
     &               * evec(j,l,ms,state%l1) * SESdipatm(i,state%l2,j,state%l1)   &
     &               * cmplx(sqrt(  real(2 * state%l1 + 1, dblprec)            &
     &                            / real(2 * state%l2 + 1, dblprec))           &
     &                       * clebsch(state%l1,1,state%l2, ms,0,ms)           &
     &                       * clebsch(state%l1,1,state%l2, 0,0,0), 0.0D0, dblprec)
              end do
            end do
          end do
        end do
      end do
      end subroutine calctwotrans


!
! clebsch --- alternate interface to the anglib Clebsch-Gordon
!       coefficients function
!

      real(dblprec) function clebsch(j1,j2,j,m1,m2,m)
      use anglib
      implicit none

      integer, intent(in) :: j1, m1, j2, m2, j, m
      integer j1_al, m1_al, j2_al, m2_al, j_al, m_al

      j1_al = 2 * j1 ; m1_al = 2 * m1
      j2_al = 2 * j2 ; m2_al = 2 * m2
      j_al  = 2 * j  ; m_al  = 2 * m
      clebsch = cleb(j1_al, m1_al, j2_al, m2_al, j_al, m_al)
      end function clebsch
      end program twoph
